from collections.abc import Callable


def map[A, B](f: Callable[[A], B], l: list[A]) -> list[B]:
    return [f(x) for x in l]


def find[A](f: Callable[[A], bool], l: list[A]) -> A | None:
    for x in l:
        if f(x):
            return x
    return None


def filter[A](f: Callable[[A], bool], l: list[A]) -> list[A]:
    return [x for x in l if f(x)]


def exists[A](f: Callable[[A], bool], l: list[A]) -> bool:
    return any([f(x) for x in l])


def forall[A](f: Callable[[A], bool], l: list[A]) -> bool:
    return all([f(x) for x in l])


def tabulate[A](n: int, f: Callable[[int], A]) -> list[A]:
    if n == 0:
        return []
    else:
        return [*tabulate(n - 1, f), f(n - 1)]
